unit uHtmlViewer;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, IPPeerClient, IPPeerServer,
  System.Tether.Manager, System.Tether.AppProfile, Vcl.StdCtrls, Vcl.ExtCtrls
  , HTMLUn2, HtmlView, System.ImageList, Vcl.ImgList, System.Actions,
  Vcl.ActnList, Vcl.Buttons, FramView, Vcl.ComCtrls;

type
  TfrmViewer1 = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    btnGo: TButton;
    TetheringAppProfile1: TTetheringAppProfile;
    TetheringManager1: TTetheringManager;
    Memo1: TMemo;
    pnlHeader: TPanel;
    HtmlViewer1: THtmlViewer;
    btnBack: TBitBtn;
    btnForward: TBitBtn;
    ActionList1: TActionList;
    actConnect: TAction;
    actGo: TAction;
    actGoBack: TAction;
    actGoForward: TAction;
    ImageList1: TImageList;
    btnConnect: TButton;
    Shape1: TShape;
    cboURL: TComboBox;
    sbLinks: TStatusBar;
//    procedure TetheringManager1EndAutoConnect(Sender: TObject);
    procedure TetheringManager1RemoteManagerShutdown(const Sender: TObject;
      const AManagerIdentifier: string);
    procedure TetheringManager1EndManagersDiscovery(const Sender: TObject;
      const ARemoteManagers: TTetheringManagerInfoList);
    procedure TetheringManager1EndProfilesDiscovery(const Sender: TObject;
      const ARemoteProfiles: TTetheringProfileInfoList);
//    procedure btnAutoConnectClick(Sender: TObject);
    procedure TetheringAppProfile1ResourceReceived(const Sender: TObject;
      const AResource: TRemoteResource);
    procedure actConnectExecute(Sender: TObject);
    procedure actGoExecute(Sender: TObject);
    procedure actGoBackExecute(Sender: TObject);
    procedure actGoForwardExecute(Sender: TObject);
    procedure HtmlViewer1HotSpotClick(Sender: TObject; const SRC: string;
      var Handled: Boolean);
    procedure HtmlViewer1HotSpotCovered(Sender: TObject; const SRC: string);
  private
    procedure Go(const aURL: string);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmViewer1: TfrmViewer1;

implementation

{$R *.dfm}

procedure TfrmViewer1.actConnectExecute(Sender: TObject);
begin
  Screen.Cursor := crHourGlass;
  TetheringManager1.DiscoverManagers;
end;

procedure TfrmViewer1.actGoBackExecute(Sender: TObject);
begin
  // Todo: TBI
  HtmlViewer1.HistoryBack;
end;

procedure TfrmViewer1.actGoExecute(Sender: TObject);
begin
  Go(cboURL.Text);
end;

procedure TfrmViewer1.actGoForwardExecute(Sender: TObject);
begin
  // Todo: TBI
  HtmlViewer1.HistoryForward;
end;

//procedure TfrmClient.btnAutoConnectClick(Sender: TObject);
//begin
//  Screen.Cursor := crHourGlass;
//  TetheringManager1.AutoConnect();
//end;

procedure TfrmViewer1.TetheringManager1EndManagersDiscovery(const Sender: TObject;
  const ARemoteManagers: TTetheringManagerInfoList);
var
  lRemoteManager: TTetheringManagerInfo;
  lManagerFound: Boolean;
begin
  lManagerFound := false;
  for lRemoteManager in ARemoteManagers do
    if (lRemoteManager.ManagerText = 'TetheringManager1') then
    begin
      TetheringManager1.PairManager(lRemoteManager);
      lManagerFound := true;
      OutputDebugString('TetheringManager1 paired');
    end;
  if not lManagerFound then
  begin
    Screen.Cursor := crDefault;
    OutputDebugString('no remote managers found');
  end;
end;

procedure TfrmViewer1.TetheringManager1EndProfilesDiscovery(const Sender: TObject;
  const ARemoteProfiles: TTetheringProfileInfoList);
var
  lRemoteProfile: TTetheringProfileInfo;
begin
  Screen.Cursor := crDefault;
  for lRemoteProfile in ARemoteProfiles do
    if lRemoteProfile.ProfileText = 'TetheringAppProfile1' then
    begin
      TetheringAppProfile1.Connect(lRemoteProfile);
      OutputDebugString('TetheringAppProfile1 connected to remote profile');
      Shape1.Brush.Color := clLime;
      btnConnect.Caption := 'Connected';
      btnConnect.Enabled := False;
    end;
end;

procedure TfrmViewer1.TetheringManager1RemoteManagerShutdown(const Sender: TObject;
  const AManagerIdentifier: string);
begin
  TetheringAppProfile1.Disconnect(TetheringAppProfile1.GetConnectedProfiles[0]);
  Shape1.Brush.Color := clWhite;
  btnConnect.Caption := 'Connect';
  btnConnect.Enabled := True;
end;

procedure TfrmViewer1.HtmlViewer1HotSpotClick(Sender: TObject; const SRC: string;
  var Handled: Boolean);
begin
  Go(SRC);
  Handled := true; // stop the processing here
end;

procedure TfrmViewer1.HtmlViewer1HotSpotCovered(Sender: TObject;
  const SRC: string);
begin
  sbLinks.Panels[0].Text := SRC;
end;

procedure TfrmViewer1.Go(const aURL: string);
begin
  if (TetheringManager1.RemoteManagers.Count = 0) then
    raise Exception.Create('not connected; connect first');
  if not TetheringAppProfile1.SendString(TetheringManager1.RemoteProfiles.Items[0], 'URL', aURL) then
    raise Exception.Create('cannot send');
end;

// from http://www.devsuperpage.com/Articles/views/Delphi/Art_1-2146.asp
//procedure LoadStream(WebBrowser: TWebBrowser; Stream: TStream);
//var
//  PersistStreamInit: IPersistStreamInit;
//  StreamAdapter: IStream;
//  MemoryStream: TMemoryStream;
//begin
//  {Load empty HTML document into Webbrowser to make "Document" a valid HTML document}
//  WebBrowser.Navigate('about:blank');
//  {wait until finished loading}
//  repeat
//    Application.ProcessMessages;
//    Sleep(0);
//  until
//    WebBrowser.ReadyState = READYSTATE_COMPLETE;
//  {Get IPersistStreamInit - interface}
//  if WebBrowser.Document.QueryInterface(IPersistStreamInit, PersistStreamInit) =
//S_OK then
//  begin
//    {Clear document}
//    if PersistStreamInit.InitNew = S_OK then
//    begin
//      {Make local copy of the contents of Stream if you want to use Stream directly,
//			you have to consider, that StreamAdapter will destroy it automatically}
//      MemoryStream := TMemoryStream.Create;
//      try
//        MemoryStream.CopyFrom(Stream, 0);
//        MemoryStream.Position := 0;
//      except
//        MemoryStream.Free;
//        raise;
//      end;
//      {Use Stream-Adapter to get IStream interface to our stream}
//      StreamAdapter := TStreamAdapter.Create(MemoryStream, soOwned);
//      {Load data from Stream into WebBrowser}
//      PersistStreamInit.Load(StreamAdapter);
//    end;
//  end;
//end;

procedure TfrmViewer1.TetheringAppProfile1ResourceReceived(const Sender: TObject;
  const AResource: TRemoteResource);
var
  SS: TStringStream;
  s: string;
begin
  if AResource.Hint = 'Response' then
  begin
    SS := TStringStream.Create;
    try
      SS.LoadFromStream(AResource.Value.AsStream);
      HtmlViewer1.LoadFromStream(SS);
    finally
      SS.Free;
    end;
  end;
end;

//procedure TfrmClient.TetheringManager1EndAutoConnect(Sender: TObject);
//begin
//  Shape1.Brush.Color := clLime;
//  btnConnect.Caption := 'Connected';
//  btnConnect.Enabled := False;
//  Screen.Cursor := crDefault;
//end;

end.
